"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const architect_1 = require("@angular-devkit/architect");
const core_1 = require("@angular-devkit/core");
const node_1 = require("@angular-devkit/core/node");
// tslint:disable-next-line:nx-enforce-module-boundaries
const core_2 = require("@mockly/core");
const path_1 = require("path");
const tsNode = require("ts-node");
exports.default = architect_1.createBuilder(_commandBuilder);
function _commandBuilder(options, context) {
    return __awaiter(this, void 0, void 0, function* () {
        context.reportStatus(`Executing Mockly...`);
        const sourceRoot = yield getSourceRoot(context);
        const targetOptions = yield context.getTargetOptions(context.target);
        const appDirectory = path_1.join(context.workspaceRoot, sourceRoot);
        const tsconfig = path_1.join(context.workspaceRoot, targetOptions.tsConfig);
        tsNode.register({
            project: path_1.join(context.workspaceRoot, targetOptions.tsConfig)
        });
        context.reportProgress(0, 8, 'Reading configuration');
        const config = yield getAndValidateConfiguration();
        context.reportProgress(1, 8, 'Reading resource files');
        const definitions = yield core_2.getResourcesAndDataDefinitions(config.resourceFilesGlob, appDirectory);
        context.logger.debug('definitions', definitions);
        context.reportProgress(2, 8, 'Reading rewrites files');
        const rewrites = yield getAndValidateRewritesConfiguration(config.rewritesFilesGlob, appDirectory);
        context.logger.debug('rewrites', rewrites);
        context.reportProgress(3, 8, 'Reading responses configuration files');
        const responsesConfig = yield getAndValidateResponsesConfiguration(config.responsesConfigGlob, appDirectory);
        context.logger.debug('responsesConfig', responsesConfig);
        context.reportProgress(4, 8, 'Reading custom controller files');
        const customControllers = yield core_2.getCustomControllers(config.customControllersGlob, appDirectory);
        context.logger.debug('customControllers', customControllers);
        context.reportProgress(5, 8, 'Creating and hydrating databases');
        const dbsMap = yield getDatabasesMap(definitions, responsesConfig);
        context.reportProgress(6, 8, 'Creating controllers');
        const controllers = core_2.getControllers(definitions, config.prefix);
        context.reportProgress(7, 8, 'Starting the server');
        yield core_2.start([...customControllers, ...controllers], dbsMap, config, rewrites);
        context.reportProgress(8, 8, 'Mockly started');
        context.reportStatus('DONE!');
        return new Promise(() => { });
    });
}
function getAndValidateConfiguration() {
    return __awaiter(this, void 0, void 0, function* () {
        const config = yield core_2.getConfiguration();
        const errors = yield core_2.getConfigurationValidationErrors(config);
        if (errors.length === 0) {
            return config;
        }
        else {
            // this.error('Invalid configuration file!');
            return config;
        }
    });
}
function getAndValidateResponsesConfiguration(glob, currentWorkingDirectory) {
    return __awaiter(this, void 0, void 0, function* () {
        const config = yield core_2.getResponsesConfiguration(glob, currentWorkingDirectory);
        const errors = yield core_2.getResponsesConfigurationErrors(config);
        if (errors.length === 0 || errors.every(arr => arr.length === 0)) {
            return config;
        }
        else {
            // this.error('Invalid responses configuration file!');
            return config;
        }
    });
}
function getAndValidateRewritesConfiguration(glob, currentWorkingDirectory) {
    return __awaiter(this, void 0, void 0, function* () {
        const rewrites = yield yield core_2.getRewrites(glob, currentWorkingDirectory);
        const errors = yield core_2.getRewritesConfigurationErrors(rewrites);
        if (errors.length === 0 || errors.every(arr => arr.length === 0)) {
            return rewrites;
        }
        else {
            // this.error('Invalid rewrites configurations!');
            return rewrites;
        }
    });
}
function getDatabasesMap(definitions, responsesConfig) {
    return __awaiter(this, void 0, void 0, function* () {
        const restResourcesDefinitions = definitions.filter(def => def.type === core_2.ResourceType.REST_RESOURCE);
        const jsonDataDefinitions = definitions.filter(def => def.type === core_2.ResourceType.JSON_DATA);
        const resourcesDbsMap = yield core_2.createAndHydrateResourcesDatabases(restResourcesDefinitions);
        const jsonDataDb = yield core_2.createAndHydrateJsonDatabase(jsonDataDefinitions);
        const responsesConfigDb = yield core_2.createAndHydrateResponsesConfigDatabase(responsesConfig);
        resourcesDbsMap.set('data', jsonDataDb);
        resourcesDbsMap.set('responses', responsesConfigDb);
        return resourcesDbsMap;
    });
}
function getSourceRoot(context) {
    return __awaiter(this, void 0, void 0, function* () {
        const workspaceHost = core_1.workspaces.createWorkspaceHost(new node_1.NodeJsSyncHost());
        const { workspace } = yield core_1.workspaces.readWorkspace(context.workspaceRoot, workspaceHost);
        // @ts-ignore
        if (workspace.projects.get(context.target.project).sourceRoot) {
            // @ts-ignore
            return workspace.projects.get(context.target.project).sourceRoot;
        }
        else {
            context.reportStatus('Error');
            const message = `${
            // @ts-ignore
            context.target.project} does not have a sourceRoot. Please define one.`;
            context.logger.error(message);
            throw new Error(message);
        }
    });
}
//# sourceMappingURL=index.js.map