"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const schematics_1 = require("@angular-devkit/schematics");
const core_1 = require("@angular-devkit/core");
const path_1 = require("path");
const workspace_1 = require("../../utils/workspace");
function app(_options) {
    return (tree, _context) => {
        let parentFolder = null;
        if (_options.path !== null &&
            _options.path !== undefined &&
            _options.path.length !== 0) {
            parentFolder = _options.path;
        }
        else {
            const workspaceConfig = workspace_1.getWorkspaceConfig(tree);
            parentFolder = workspaceConfig.newProjectRoot || '';
        }
        const appPath = path_1.join(core_1.normalize(parentFolder), _options.name);
        return schematics_1.chain([
            addMocklyAppToWorkspace(_options.name, appPath),
            createMocklyAppFiles(appPath)
        ]);
    };
}
exports.app = app;
function createMocklyAppFiles(path) {
    const templateSource = schematics_1.apply(schematics_1.url('./files'), [
        schematics_1.applyTemplates({}),
        schematics_1.move(path)
    ]);
    return schematics_1.mergeWith(templateSource);
}
function addMocklyAppToWorkspace(name, path) {
    const project = createProjectDefinition(path);
    return workspace_1.updateWorkspace(workspace => {
        workspace.projects.add(Object.assign({ name }, project));
    });
}
function createProjectDefinition(path) {
    return {
        root: path,
        sourceRoot: `${path}/src`,
        projectType: 'application',
        architect: {
            serve: {
                builder: '@mockly/angular:start',
                options: {
                    tsConfig: `${path}/tsconfig.json`
                }
            }
        }
    };
}
//# sourceMappingURL=index.js.map