"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Utilities from @nrwl/workspace.
 *
 * Original code can be found in the following links:
 *
 * https://github.com/nrwl/nx/blob/master/packages/workspace/src/utils/fileutils.ts
 * https://github.com/nrwl/nx/blob/master/packages/workspace/src/utils/workspace.ts
 */
const stripJsonComments = require('strip-json-comments');
function updateJsonInTree(path, callback) {
    return (host, context) => {
        if (!host.exists(path)) {
            host.create(path, serializeJson(callback({}, context)));
            return host;
        }
        host.overwrite(path, serializeJson(callback(readJsonInTree(host, path), context)));
        return host;
    };
}
exports.updateJsonInTree = updateJsonInTree;
function readJsonInTree(host, path) {
    if (host === null || host === undefined) {
        throw new Error('Host not available!');
    }
    if (!host.exists(path)) {
        throw new Error(`Cannot find ${path}`);
    }
    // tslint:disable-next-line:no-non-null-assertion
    const contents = stripJsonComments(host.read(path).toString('utf-8'));
    try {
        return JSON.parse(contents);
    }
    catch (e) {
        throw new Error(`Cannot parse ${path}: ${e.message}`);
    }
}
exports.readJsonInTree = readJsonInTree;
function serializeJson(json) {
    return `${JSON.stringify(json, null, 2)}\n`;
}
exports.serializeJson = serializeJson;
//# sourceMappingURL=files.js.map