"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@angular-devkit/core");
const schematics_1 = require("@angular-devkit/schematics");
/**
 * Utilities from @nrwl/workspace.
 *
 * Original code can be found in the following links:
 *
 * https://github.com/nrwl/nx/blob/master/packages/workspace/src/utils/fileutils.ts
 * https://github.com/nrwl/nx/blob/master/packages/workspace/src/utils/workspace.ts
 */
function updateWorkspace(updater) {
    return (tree) => __awaiter(this, void 0, void 0, function* () {
        const host = createHost(tree);
        const { workspace } = yield core_1.workspaces.readWorkspace('/', host);
        const result = updater(workspace);
        if (result !== null && result !== undefined) {
            yield result;
        }
        yield core_1.workspaces.writeWorkspace(workspace, host);
    });
}
exports.updateWorkspace = updateWorkspace;
function getWorkspaceConfig(tree) {
    const workspaceConfig = tree.read('/angular.json');
    if (!workspaceConfig) {
        throw new schematics_1.SchematicsException('Could not find Angular workspace configuration');
    }
    // convert workspace to string
    const workspaceContent = workspaceConfig.toString();
    // parse workspace string into JSON object
    const workspace = JSON.parse(workspaceContent);
    return workspace;
}
exports.getWorkspaceConfig = getWorkspaceConfig;
function createHost(tree) {
    return {
        readFile(path) {
            return __awaiter(this, void 0, void 0, function* () {
                const data = tree.read(path);
                if (!data) {
                    throw new Error('File not found.');
                }
                return data.toString();
            });
        },
        writeFile(path, data) {
            return __awaiter(this, void 0, void 0, function* () {
                return tree.overwrite(path, data);
            });
        },
        isDirectory(path) {
            return __awaiter(this, void 0, void 0, function* () {
                // approximate a directory check
                return !tree.exists(path) && tree.getDir(path).subfiles.length > 0;
            });
        },
        isFile(path) {
            return __awaiter(this, void 0, void 0, function* () {
                return tree.exists(path);
            });
        }
    };
}
//# sourceMappingURL=workspace.js.map